#include <windows.h>
#include <pnp.h>
#include <stdio.h>
#include "..\inc\EAU.h" //for shared guid... could be read from registry

DWORD WINAPI FindDeviceThread(LPVOID lpParm);
BOOL AccessTest(PTCHAR pszDeviceName);

#define APP_NAME L"EAU Test "

int
WasOption
// look for argument like '-t' or '/t'.
// returns option index
// returns index of argv[] found, -1 if not found
(
    IN int argc,       // number of args
    IN WCHAR* argv[],  // arg array
    IN WCHAR* szOption // to find ('t')
)
{
    for(int i=0; i<argc; i++)
    {
        if( ((*argv[i] == L'-') || (*argv[i] == L'/')) &&
            !wcscmp(argv[i]+1, szOption))
            return i;
    }
    return -1;
}   // WasOption()



// look for argument like '-t 100' or '/t 100'.
// returns index of '100' if option ('t') is found
// returns -1 if not found
int
GetOption(
    int argc,                // number of args
    IN WCHAR* argv[],        // arg array
    IN WCHAR* pszOption,     // to find ('n')
    OUT WCHAR** ppszArgument // option value ('100')
)
{
    if(!ppszArgument)
        return -1;

    int i = WasOption(argc, argv, pszOption);
    if((i < 0) || ((i+1) >= argc))
    {
        *ppszArgument = NULL;
        return -1;
    }

    *ppszArgument = argv[i+1];
    return i+1;
}   // GetOption()




void wmain(
	ULONG	argc,
	TCHAR  *argv[])
{
	const TCHAR szVersion[] = __WINCEOSVERSION__;
    TCHAR szDeviceName[] = L"EAU1:";
	WCHAR *pOption;
	int LoopCount;
	HANDLE hThread;


	printf("%s - OS Version %s\r\n",APP_NAME, szVersion);
	RETAILMSG(1,(TEXT("%s App - OS Version %s\r\n"),APP_NAME, szVersion));
	if (GetOption(argc, argv, _T("n"), &pOption) >= 0) {
		LoopCount = _wtoi(pOption);
		if (LoopCount < 2)
			LoopCount=2;
	}

	if (LoopCount) {
	printf("%s - Test using Device Notifications %d\r\n",APP_NAME, LoopCount);
	RETAILMSG(1,(TEXT("%s App - Test using Device Notifications %d\r\n"),APP_NAME, LoopCount));

		hThread = CreateThread(NULL, 0, FindDeviceThread, (LPVOID)LoopCount, 0, NULL);
		WaitForSingleObject(hThread, INFINITE);
	} else {
		AccessTest((PTCHAR)&szDeviceName);
	}
    return;
}

//GUID Mass_guid = {0xA4E7EDDA, 0xE575, 0x4252, {0x9D, 0x6B, 0x41, 0x95, 0xD4, 0x8B, 0xB8, 0x65}};    // or any known and relevant device interface GUID
//GUID NDIS_guid = {0x98C5250D, 0xC29A, 0x4985, {0xAE, 0x5F, 0xAF, 0xE5, 0x36, 0x7E, 0x50, 0x06}};    // or any known and relevant device interface GUID
//GUID Serial_guid = {0xCC5195AC, 0xBA49, 0x48a0, {0xBE, 0x17, 0xDF, 0x6D, 0x1B, 0x01, 0x73, 0xDD}};
DWORD WINAPI FindDeviceThread( LPVOID lpParm)
{
 	BYTE pPNPBuf[sizeof(DEVDETAIL) + MAX_DEVCLASS_NAMELEN * sizeof(TCHAR)] = {0};
	DEVDETAIL *pd = (DEVDETAIL*)pPNPBuf;
	HANDLE hq, hn;
	MSGQUEUEOPTIONS msgopts;
	MSGQUEUEINFO msginfo = {0};
	DWORD	 LoopCount = (DWORD)lpParm;
	DWORD	 count ;




	msgopts.dwFlags = MSGQUEUE_ALLOW_BROKEN | MSGQUEUE_NOPRECOMMIT;
	msgopts.cbMaxMessage = sizeof(pPNPBuf);
	msgopts.dwSize = sizeof(msgopts);
	msgopts.dwMaxMessages = 10;
	msgopts.bReadAccess = true;
	hq = CreateMsgQueue(NULL, &msgopts);
	if (hq) {
		hn = RequestDeviceNotifications(&EAU_guid, hq, TRUE);
		DWORD flags;
		DWORD size;

		SetLastError(0);

		msginfo.dwSize = sizeof(msginfo);
		count = 1;
		if (GetMsgQueueInfo(hq ,&msginfo)) {
			count = msginfo.dwCurrentMessages;
		}

		count = LoopCount;
		while (ReadMsgQueue(hq, pd, sizeof(pPNPBuf), &size, INFINITE, &flags) == TRUE) {

			printf("device \"%S\" changed !!!!\n",pd->szName);
			if (pd->fAttached) {
				RETAILMSG(1,(TEXT("Device \"%s\" ATTACHED!!!!!!!!\n"),pd->szName));
				printf(" device \"%S\" attached\n",pd->szName);
				AccessTest((PTCHAR)&pd->szName);

			} else {
				RETAILMSG(1,(TEXT("Device \"%s\" DEATACHED********\n"),pd->szName));
				printf(" device \"%S\" detached\n",pd->szName);
			}

			count--;
			if (count <= 0)
				break;
		}

		StopDeviceNotifications(hn);

		CloseMsgQueue(hq);
	}
	RETAILMSG(1,(TEXT("%s Exiting the notification thread\r\n"),APP_NAME));
	return S_OK;

}


DWORD GenericVendorRead(HANDLE hDevice, BYTE Request, WORD Value, WORD Index, DWORD & DataSize, LPVOID pData)
{
	VENDOR_REQUEST_DATA VRD;
	DWORD Result;

	VRD.Request = Request;
	VRD.Value = Value;
	VRD.Index = Index;
	if ( DeviceIoControl(hDevice, IOCTL_EAU_READ, (LPVOID)&VRD, sizeof(VRD), pData, DataSize, &DataSize, NULL) )
		return ERROR_SUCCESS;
	else
	{
		DataSize = 0;
		Result = GetLastError();
		if ( Result )
			return Result;
		else
			return ERROR_GEN_FAILURE
		;
	}
}

DWORD GenericVendorWrite(HANDLE hDevice, BYTE Request, WORD Value, WORD Index, DWORD & DataSize, PVOID pData)
{
	DWORD IOSize;
	PBYTE pBuf;
	PVENDOR_REQUEST_DATA pVRD;
	DWORD Result;

	IOSize = sizeof(*pVRD) + DataSize;
	pVRD = PVENDOR_REQUEST_DATA(malloc(IOSize));
	pBuf = PBYTE(pVRD + 1);
	pVRD->Request = Request;
	pVRD->Value = Value;
	pVRD->Index = Index;
	if ( DataSize > 0 ) memmove(pBuf, pData, DataSize);
	if ( DeviceIoControl(hDevice, IOCTL_EAU_WRITE, (LPVOID)pVRD, IOSize, &DataSize, 1, NULL, NULL) )
		return ERROR_SUCCESS;
	else
	{
		DataSize = 0;
		Result = GetLastError();
		if ( Result )
			return Result;
		else
			return ERROR_GEN_FAILURE
		;
	}
}

BOOL AccessTest(PTCHAR pDeviceName)
{
	HANDLE	hDev;
	DWORD	loop;
	BOOL	Result;
	DWORD	err;

	DWORD	DIOData;
	DWORD	Len;

	printf("Opening %S... ", pDeviceName);
	hDev = CreateFile (pDeviceName, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0);
	// open driver
	if (hDev == INVALID_HANDLE_VALUE)
	{
		err= GetLastError();
		printf("Fail, error %d %x \r\n", err, err);
		RETAILMSG(1,(TEXT("Failed to open %s, error %d %x\r\n"),pDeviceName,err, err));
		return FALSE;
	}
	else
	{
		printf("Success.\r\n");
	}

	for ( loop = 0; loop <= 15; ++loop )
	{
		Len = 4;
		printf("GenericVendorRead... ");
		err = GenericVendorRead(hDev, 0x11, 0, 0, Len, &DIOData);
		if ( err )
		{
			printf("Fail, error %d %x\r\n",err, err);
			RETAILMSG(1,(TEXT("GenericVendorRead Failed, error %d %x\r\n"),err, err));
			Result = FALSE;
		}
		else
		{
			if ( Len >= 4 )
				printf("Success, DIOData = %08X\r\n", DIOData);
			else
				printf("Success, but returned %d bytes\r\n", Len)
			;
		}

		Len = 4;
		DIOData = 1 << loop;
		printf("GenericVendorWrite(%08X)... ", DIOData);
		err = GenericVendorWrite(hDev, 0x10, 0, 0, Len, &DIOData);
		if ( err )
		{
			printf("Fail, error %d %x\r\n",err, err);
			RETAILMSG(1,(TEXT("GenericVendorWrite failed, error %d %x\r\n"),err, err));
			Result = FALSE;
		}
		else
		{
			printf("Success, wrote %d bytes\r\n",Len);
		}
		Sleep(500);
	}

	printf("Closing %S\r\n",pDeviceName);
	CloseHandle(hDev);

	return Result;
}
